// file:~kaldindi/work/class/ppm/image_analysis.cc
//

// system include files
//
#include "ppm.h"
#include <math/integral_constants.h>

#include <string.h>
#include <stdlib.h>

// this method performs image analysis methods such as histogram,
// edge detection and outputs the scenic beauty

main(int argc, char** argv) {
  
  if (argc != 2) {
    fprintf(stdout, "%s", "give proper arguments");
  }
  
  // declare an object
  //
  Ppm ppm1,ppm2,ppm3,ppm4,ppm5,ppm6;
  
  // open the input file
  //
  if (ppm1.open_cc((char_1*)argv[1],ISIP_READ_ONLY) == ISIP_FALSE) {
    return ISIP_ERROR;
  }
  
  char filename_pgm[ISIP_MAX_FNAME_SIZE];
  char filename_hist[ISIP_MAX_FNAME_SIZE];
  char filename_edge[ISIP_MAX_FNAME_SIZE];
  char filename_ln[ISIP_MAX_FNAME_SIZE];
  char filename_aly[ISIP_MAX_FNAME_SIZE];

  // assign the .pgm extension for the gray scale
  //
  strcpy(filename_pgm, strtok(argv[1],"."));
  strcat(filename_pgm,".pgm");
  
  // create the output file
  //
  if (ppm2.open_cc((char_1*)filename_pgm, ISIP_WRITE_PLUS)
      == ISIP_FALSE) {
    return ISIP_ERROR;
  }

  // assign the .hist extension for the histogram file
  //
  strcpy(filename_hist, strtok(argv[1], "."));
  strcat(filename_hist,".hist");
  
  // create the output file
  //
  if (ppm3.open_cc((char_1*)filename_hist, ISIP_WRITE_ONLY)
      == ISIP_FALSE) {
    return ISIP_ERROR;
  }
  
  // assign the edge.pgm extension for the edge detection
  //
  strcpy(filename_edge, strtok(argv[1], "."));
  strcat(filename_edge, "edge.pgm");
  
  if (ppm4.open_cc((char_1*)filename_edge, ISIP_WRITE_ONLY)
      == ISIP_FALSE) {
    return ISIP_ERROR;
  }
  
  // assign the .ln extension for the vertical lines file
  //
  strcpy(filename_ln, strtok(argv[1], "."));
  strcat(filename_ln, ".ln");
  
  if (ppm5.open_cc((char_1*)filename_ln, ISIP_WRITE_ONLY)
      == ISIP_FALSE) {
    return ISIP_ERROR;
  }
  
  int_4 width = ppm1.get_width_cc();
  int_4 height = ppm1.get_height_cc();
  int_4 max_color = ppm1.get_max_colors_cc();
  int_4 bsize = 4;
  int_4 total_size = width * height;
  
  // define a block size and associated buffers
  //
  uint_1* data = new uint_1[total_size];
  uint_1* pixel_value = new uint_1[bsize];
  uint_1* b_red = new uint_1[bsize*bsize];
  uint_1* b_green = new uint_1[bsize*bsize];
  uint_1* b_blue = new uint_1[bsize*bsize];
  
  int_4* redcount = new int_4[256];
  int_4* greencount = new int_4[256];
  int_4* bluecount = new int_4[256];
  
  for(int_2 l =0; l<256; l++) {
    redcount[l] = 0;
    greencount[l] = 0;
    bluecount[l] = 0;
  }
  
  if (ppm2.write_colorheader_cc(width, height, max_color) ==
      ISIP_FALSE) {
    return ISIP_ERROR;
  }

  for (int_4 i=0; i<height; i+=bsize) {
    for (int_4 j=0; j<width; j+=bsize) {
      
      // reading the data form the image
      //
      if (ppm1.read_cc(b_red, b_green, b_blue,
		       j, i, bsize, bsize) == ISIP_FALSE) {
	fprintf(stdout, "%s: error reading the image\n", argv[0]);
      }

      // converting it to gray scale
      //
      if (ppm2.gray_cc(pixel_value, j, i,bsize, bsize ,
		       b_red, b_green, b_blue) == ISIP_FALSE) {
	fprintf(stdout, "%s: error from color to gray leveal\n",argv[0]);
      }

      // writing to the pgm file
      //
      if (ppm2.writepgm_cc(pixel_value, j, i, bsize,
			   bsize) == ISIP_FALSE) {
	fprintf(stdout, "%s: error writing the pgm file\n", argv[0]);
      }

      // computing the mean of the colors
      //
      if (ppm3.mean_cc(b_red, b_green, b_blue, bsize, redcount,
		       greencount, bluecount) == ISIP_FALSE) {
	fprintf(stdout,"%s: error in computing the mean",argv[0]);
      }
    }
  }
  
  int_4 redmean = 0;
  int_4 greenmean = 0;
  int_4 bluemean = 0;
  
  for(int_4 index = 0; index < 255; index++)
    {
      redmean = redmean + redcount[index] * index ;
      greenmean = greenmean + greencount[index] * index;
      bluemean = bluemean + bluecount[index] * index;
    }
  
  redmean = redmean / (width * height);
  greenmean = greenmean / (width * height);
  bluemean = bluemean / (width * height);
  
  fprintf(stdout,"%ld %ld %ld\n", redmean, greenmean, bluemean);
  
  if (ppm3.writehist_cc(redcount, greencount, bluecount,redmean, greenmean,
			bluemean, max_color) == ISIP_FALSE) {
    fprintf(stdout,"%s: error writing the hist file",argv[0]);
  }
  
  if(ppm2.readpgm_cc(data, width, height) == ISIP_FALSE) {
    fprintf(stdout,"%s: error reading the pgm file",argv[0]);
  }
  
  ppm3.close_cc();
  
  if (ppm3.open_cc((char_1*)filename_pgm, ISIP_READ_ONLY) == ISIP_FALSE) {
    fprintf(stdout, "%s: error opening the file",argv[0]);
  }
  
  if (ppm3.readpgm_cc(data, width, height) == ISIP_FALSE) {
  fprintf(stdout,"%s: error reading the pgm file",argv[0]);
  }

  // edge detection using canny-edge detector algorithm
  //
  if (ppm4.edge_cc(data, width, height) == ISIP_FALSE) {
    fprintf(stdout, "%s: error converting to edges", argv[0]);
    return ISIP_ERROR;
  }
  
  if (ppm4.write_colorheader_cc(width, height, max_color) ==
      ISIP_FALSE) {
    return ISIP_ERROR;
  }
  
  if (ppm4.writepgm_cc(data, 0, 0, width, height) == ISIP_FALSE) {
    fprintf(stdout, "%s: error writing the edge detected file",argv[0]);
    return ISIP_ERROR;
  }
  
  int_4* lines = new int_4[width*height];
  
  for (int_4 l=0; l< width*height; l++) {
    lines[l] = 0;
  }
  
  uint_2* condata = new uint_2[width*height];
  
  // converting the unsigned char to unsigned int
  //
  if (ppm5.condata_cc(condata, data, width, height) == ISIP_FALSE) {
    fprintf(stdout,"%s: error converting the data to int",argv[0]);
  }
  
  int_4* max_line = new int_4[1];

  // computing the number of vertical lines
  //
  if (ppm5.line_cc(lines, condata, width, height, max_line) == ISIP_FALSE) {
    fprintf(stdout, "%s: error computing the vertical lines\n", argv[0]);
  }
  
  if (ppm5.writeline_cc(lines, height) == ISIP_FALSE) {
    fprintf(stdout,"%s: error in writing the lines to the file",argv[0]);
  }
  
  ppm1.close_cc();
  ppm2.close_cc();
  ppm3.close_cc();
  ppm4.close_cc();
  ppm5.close_cc();
  
  // evaluating the color values
  //
  int_4 total = redmean*redmean + greenmean*greenmean + bluemean*bluemean;
  float_4 result = sqrt(total);
  
  float_4 red_mean = redmean / result;
  float_4 green_mean = greenmean / result;
  float_4 blue_mean = bluemean / result;
  
  int_4 small_lines = 0;
  
  for (int_2 b=0; b< 25; b++) {
    small_lines = small_lines + lines[b];
  }
  
  int_4 long_lines = 0;
  
  for (int_2 c=50; c<height; c++) {
    long_lines = long_lines + lines[c];
  }
  
  float_4 percent_small = small_lines / max_line[0];
  float_4 percent_long = long_lines / max_line[0];
  
  float_4 sbe_value = 0;
  
  // doing the image_analysis
  //
  if (percent_long > 0.02) {
    sbe_value = 1.1*green_mean - 0.4*red_mean - 0.1*blue_mean +
      7*percent_long - 0.2*percent_small;
  }
  
  else if (percent_long < 0.013) {
    sbe_value = 1.1*green_mean - 0.4*red_mean - 0.1*blue_mean +
      10*percent_long - 0.2*percent_small;
  }
  
  else {
    sbe_value = 1.1*green_mean - 0.4*red_mean - 0.1*blue_mean +
      8*percent_long - 0.2*percent_small;
  }
  
  strcpy(filename_aly, strtok(argv[1],"."));
  strcat(filename_aly,".aly");
  
  if (ppm6.open_cc((char_1*)filename_aly,ISIP_WRITE_ONLY) == ISIP_FALSE) {
    fprintf(stdout,"%s: error opening the aly file",argv[0]);
  }
  
  if(ppm6.writesbe_cc(sbe_value) == ISIP_FALSE) {
    fprintf(stdout,"%s: error while writing the file",argv[0]);
  }
  ppm6.close_cc();
}



