// file: ~kaldindi/work/class/ppm/ppm.h
//	

// make sure definitions are made only once
//
#ifndef __ISIP_PPM
#define __ISIP_PPM

// isip include files
//
#include <math/integral.h>

// Ppm : a class meant to handle the portable pixel image file format (ppm)
//
class Ppm {

  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:

  // filename information
  //
  char_1* fname_d;
  FILE* fp_d;              // file pointer

  // information about the image (read from the ppm file)
  //
  char_1* magic_number_d;  // number which tells that the file is a .ppm 
  int_4 width_d;           // gives the width of the image
  int_4 height_d;          // gives the height of the image
  int_4 max_colors_d;      // gives the maximum color level in the image

  // file positioning information
  //
  int_4 beg_d;		   // beginning of file

  //---------------------------------------------------------------------------
  //
  // public methods
  //
  //---------------------------------------------------------------------------
public:

  // required methods
  //
  char_1* name_cc();
  volatile void error_handler_cc(char_1* method_name, char_1* message);
  logical_1 debug_cc(FILE* fp, char_1* message);

  // destructor/constructor
  //
  ~Ppm();
  Ppm();

  // open/close methods
  //
  logical_1 open_cc(char_1* filename, int_4 mode);
  logical_1 close_cc();

  // read/write methods
  //
  logical_1 read_header_cc();
  logical_1 read_colorheader_cc();
  
  logical_1 read_cc(uint_1* b_red, uint_1* b_green, uint_1* b_blue,
		    int_4 offset_w, int_4 offset_h, int_4 width, int_4 height);

  logical_1 readpgm_cc(uint_1* pixel_value, int_4 width, int_4 height);
  
  logical_1 write_header_cc(int_4 width, int_4 height, int_4 max_colors);

  logical_1 write_colorheader_cc(int_4 width, int_4 height, int_4 max_colors);
  
  logical_1 write_cc(uint_1* b_red, uint_1* b_green, uint_1* b_blue,
		     int_4 offset_w, int_4 offset_h,
		     int_4 width, int_4 height);

  logical_1 writepgm_cc(uint_1* pixel_value, int_4 offset_w, int_4 offset_h,
			int_4 width, int_4 height);

  logical_1 writehist_cc(int_4* redcount, int_4* greencount, int_4* bluecount,
			 int_4 redmean, int_4 greenmean, int_4 bluemean,
			 int_4 max_color);

  logical_1 writeline_cc(int_4* lines, int_4 height);
  
  // assignment methods
  //
  logical_1 assign_cc(int_4 width, int_4 height, int_4 max_colors);

  // computation methods
  //
  logical_1 gray_cc(uint_1* pixel_value, int_4 offset_w, int_4 offset_h,
		    int_4 width, int_4 height, uint_1* b_red, uint_1* b_green,
		    uint_1* b_blue);
  

  logical_1 mean_cc(uint_1* b_red, uint_1* b_green, uint_1* b_blue,
		    int_4 bsize, int_4* redcount, int_4* greencount,
		    int_4* bluecount);

  logical_1 cannycore_cc(float_8 s, int_4 width, int_4 height, uint_1* data,
			 uint_1* derivative_mag, uint_1* magnitude,
			 uint_1* orientation);
  
  float_8 gaussian_cc(float_8 x, float_8 s);

  float_8 hypotenuse_cc(float_8 x, float_8 y);
  
  logical_1 thresholding_cc(int_2 high, int_2 low, int_4 cols, int_4 rows,
			    uint_1* data, uint_1* magnitude,
			    uint_1* orientation);

  logical_1 follow_cc(int_2 i, int_2 j, int_2 low, int_4 cols, int_4 rows,
		      uint_1* data, uint_1* magnitude, uint_1* orientation);

  logical_1 pgmcanny_cc(char* infile, char* outfile);
  
  logical_1 line_cc(int_4* lines, uint_2* condata,
		    int_4 width, int_4 height, int_4* max_line);

  logical_1 condata_cc(uint_2* condata, uint_1* data, int_4 width,
		       int_4 height);

  logical_1 writesbe_cc(float_4 sbe_value);

  // analysis methods
  //
  logical_1 edge_cc(uint_1* data, int_4 width, int_4 height);

  // comparison methods
  //
  logical_1 diff_cc(float_4 alpha, float_4 beta,
		    int_4 width, int_4 height,
		    uint_1* b_red, uint_1* b_green, uint_1* b_blue,
		    uint_1* b_red_0, uint_1* b_green_0, uint_1* b_blue_0,
		    uint_1* b_red_1, uint_1* b_green_1, uint_1* b_blue_1);

  // get information methods
  //
  int_4 get_width_cc();
  int_4 get_height_cc();
  int_4 get_max_colors_cc();
  float_4 getsbe_cc();
  
  //---------------------------------------------------------------------------
  //
  // private methods
  //
  //---------------------------------------------------------------------------
private:
};

// end of file
//
#endif

