// file: ~kaldindi/work/class/ppm/ppm_edge_cc
//

//isip include files
//
#include "ppm.h"
#include <math/integral.h>
#include <math/integral_constants.h>

// method: edge_cc
//
// arguments:
//  uint_1* data: (output) data after the edge detection
//  int_4 width: (input) width of the image
//  int_4 height: (input) height of the image
//
// return: a logical_1 value indicating status
//
// this method performs edge detection using canny-edge detection algorithm
//
logical_1 Ppm::edge_cc(uint_1* data, int_4 width, int_4 height) {

  float_8 s=1.0;            /* standard deviation of gaussian */
  int_4 picsize = width * height;
  int low=100,high=255;        /* tracker hysteresis parameters */
  unsigned char *derivative_mag; /*mag of del G before non-maximum suppression*/
  unsigned char *magnitude,*orientation; /*mag & orient after non-max sppression*/
  
  
  if ((derivative_mag=(unsigned char *)calloc(picsize,1))==
      (unsigned char *)NULL)
    {
      fprintf(stderr,"canny: can't alloc derivative_mag\n");
      exit(0);
    }
  if ((magnitude=(unsigned char *)calloc(picsize,1))
      ==(unsigned char *)NULL)
    {
      fprintf(stderr,"canny: can't alloc magnitude\n");
      exit(0);
    }
  if ((orientation=(unsigned char *)calloc(picsize,1))
      ==(unsigned char *)NULL)
    {
      fprintf(stderr,"canny: can't alloc orientation\n");
      exit(0);
    }
  
  /* call canny core routines - these perform a gaussian
     smoothing, calculate a gradient array and suppress
     non- maximal points in this array */
  if (cannycore_cc(s,width,height,data,derivative_mag,magnitude,orientation)
      == ISIP_FALSE) {
    //    fprintf(stdout,"%s: error computing cannycore",argv[0]);
  }
  
  fprintf(stderr,"canny: about to track edges\n");
  /* track edges */
  if (thresholding_cc(high,low,width,height,data,magnitude,orientation)
      == ISIP_FALSE) {
    //fprintf(stdout,"%s: error doing the thresholding",argv[0]);
  }
  
  free(magnitude);
  free(orientation);

  // exit gracefully
  //
  return ISIP_TRUE;
}


