// file: ~kaldindi/work/class/ppm/gray_cc
//
// isip include files
//
#include <math/integral.h>
#include <math/integral_constants.h>
#include "ppm.h"

// arguments:
//  uint_1* pixel_value: (output) unsigned char value for pgm image
//  int_4 offset_w_a: (input) offset in the horizontal direction
//  int_4 offset_h_a: (input) offset in the vertical direction
//  int_4 width_a: (input) width of the image
//  int_4 height_a: (input) height of the image
//  uint_1* b_red_a: (input) red buffer of the ppm image
//  uint_1* b_green_a: (input) green buffer of the ppm image
//  uint_1* b_blue_a: (input) blue buffer of the ppm image
//
// return: a logical_1 value indicating status
//
// this method converts the color values into 
// an equivalent gray color value
//
logical_1 Ppm::gray_cc(uint_1* pixel_value, int_4 offset_w_a,
				  int_4 offset_h_a, int_4 height_a,
				  int_4 width_a, uint_1* b_red_a,
				  uint_1* b_green_a, uint_1* b_blue_a) {

 // check for out of bounds data in the width dimension
  //
  int_4 actual_width = width_a;

  if ((offset_w_a + actual_width) >= width_d) {
    actual_width = width_d - offset_w_a; 
  }
  if (actual_width < (int_4)0) {
    return ISIP_FALSE;
  }

  // check for out of bounds data in the height dimension
  //
  int_4 actual_height = height_a;

  if ((offset_h_a + actual_height) >= height_d) {
    actual_height = height_d - offset_h_a;
  }
  if (actual_height < (int_4)0) {
    return ISIP_FALSE;
  }
  
  
  for (int_2 i = 0; i < actual_height; i++) {
    for (int_2 j = 0; j < actual_width; j++) {
      pixel_value[i*actual_width+j] = (uint_1)(0.299*(float_4)
				       b_red_a[i*actual_width+j] +
				  0.587*(float_4)b_green_a[i*actual_width+j] +
				  0.114*(float_4)b_blue_a[i*actual_width+j]);
      
      if (abs(pixel_value[i*actual_width+j]) > 255) {
	pixel_value[i*actual_width+j] = 255;
      }
    }
  }
  // exit gracefully
  //
  return ISIP_TRUE;
}
  
  
  
