//file: ~kaldindi/work/class/ppm/ppm_readpgm_cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "ppm.h"

#include <math/integral_constants.h>
#include "ppm_constants.h"

// method: read_cc
//
// arguments:
//  uint_1* data: (output) read the data into this buffer
//  int_4 width_a: width of the image
//  int_4 height_a: height of the image
//
// return: a logical_1 value indicating status
//
// this method reads data row by row. it uses a scratch buffer
// that is large enough to hold one row worth of data.
//
logical_1 Ppm::readpgm_cc(uint_1* data, int_4 width_a,
			  int_4 height_a) {
  
    
  // loop over all rows
  //
  int_4 o_row = (int_4)0;
  int_4 last_row =  height_a;
  
  for (int_4 row_num= o_row; row_num<last_row; row_num++) {
    
    // compute the new position we need to seek to
    //
    int_4 new_pos = (row_num * width_a) + beg_d;
    
    // seek to this new location
    //
    if (fseek(fp_d, new_pos, SEEK_SET) < (int)0) {
      return ISIP_FALSE;
    }
    
    // read the data into a temporary buffer
    //
    int_4 nbytes_to_read = width_a;
    uint_1 tmp_buf[nbytes_to_read];
    
    // read the data: note that we don't check for an out of bounds
    // read. this is because many applications want to process the image
    // in sub-images that are not multiple dimensions of the image.
    //
    fread((void*)tmp_buf, nbytes_to_read, (size_t)1, fp_d);
    
    // split the data into separate buffers
    //
    uint_1* ptr = tmp_buf;
    int_4 o_offset = o_row * width_a;
    int_4 last_val = o_offset + width_a;
    
    for (int_4 i=o_offset; i<last_val; i++) {
      
      // separate each pixel
      //
      data[i] = *ptr++;
    }
    
    // increment the output row counter
    //
    o_row++;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


