// file: ~kaldindi/class/image_analysis/image_analysis_gray_0.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "ppm.h"

#include <math/integral.h>
#include "ppm_constants.h"
#include <math/integral_constants.h>

// method: writepgm_cc
//
// arguments:
// uint_1* pixel_value: (input) intensity of the pixel
// int_4 offset_w_a: (input) starting pixel in the horizontal direction
// int_4 offset_h_a: (input) starting pixel in the vertical direction
// int_4 width_a: (input) number of pixels in the horizontal direction
// int_4 height_a: (input) number of pixels in the vertical direction
//
// return: logical_1 value indicating the status
//
logical_1 Ppm::writepgm_cc(uint_1* pixel_value, int_4 offset_w_a,
			   int_4 offset_h_a, int_4 width_a,
			   int_4 height_a ) {
  
  // check for out of bounds data in the width dimension
  //
  int_4 actual_width = width_a;
  
  if ((offset_w_a + actual_width) >= width_d) {
    actual_width = width_d - offset_w_a;
  }
  
  if (actual_width < (int_4)0) {
    return ISIP_FALSE;
  }
  
  // check for out of bounds data in the height dimension
  //
  int_4 actual_height = height_a;
  
  if ((offset_h_a + actual_height) >= height_d) {
    actual_height = height_d - offset_h_a;
  }
  
  if (actual_height < (int_4)0) {
    return ISIP_FALSE;
  }
  
  
  // loop over the rows			
  //			
  int_4 o_row = (int_4)0;
  int_4 last_row = offset_h_a + actual_height;
  
  for (int_4 row_num=offset_h_a; row_num<last_row; row_num++) {
    
    // compute the new position we need to seek to
    //	
    int_4 new_pos = (row_num * width_d + offset_w_a) + beg_d;
    
    // seek to this new location
    //
    if (fseek(fp_d, new_pos, SEEK_SET) < (int)0) {
      return ISIP_FALSE;
    }
    
    // write the data first into a buffer
    //
    int_4 o_offset = o_row * width_a;
    int_4 last_val = o_offset + actual_width;
    int_4 j = 0;
    
    // read the data into a temporary buffer
    //
    uint_1* tmp_buf=new uint_1[actual_width];
    
      for (int_4 i=o_offset; i<last_val; i++) {
	tmp_buf[j] = pixel_value[i];
	j = j+1;
      }

      // write the data from the buffer into the file
      //
      if (fwrite((const void*)tmp_buf, 1, actual_width, fp_d) == 0) {
	error_handler_cc((char_1*)"fwrite",
			 (char_1*)"error in writing the message");
      }
      o_row++;
  }
  // exit gracefully
  //
  return ISIP_TRUE;
} 



















